﻿using Oxide.Core;
using Oxide.Core.Configuration;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Oxide.Plugins {
    [Info("GWhitelist", "Gokus", "0.1.0")]
    [Description("GWhitelist makes your server secure from unwanted people. Well, thats what it supposed to do anyway!")]
    class GWhitelist : HurtworldPlugin {
        DynamicConfigFile dataFile = Interface.Oxide.DataFileSystem.GetDatafile("GWhitelist");
        private void OnServerInitialized() {
            permission.RegisterPermission("gwhitelist.admin", this);
            if (dataFile["Enabled"] == null) {
                dataFile["Enabled"] = false;
                if (dataFile["AllowedSteamIds"] == null) {
                    dataFile["AllowedSteamIds"] = "76561198202253821";
                }
                dataFile.Save();
            }
        }

        protected override void LoadDefaultMessages() {
            lang.RegisterMessages(new Dictionary<string, string> {
                ["WhitelistRejectionMessage"] = "You can't access the server right now!",
                ["NoArgsProvided"] = "> You must provide valid arguments: switch, allow <steamid64> or reload!",
                ["TurnedOn"] = "> Successfully turned on the whitelist!",
                ["TurnedOff"] = "> Successfully turned off the whitelist!",
                ["SuccessfullyReloaded"] = "> Successfully reloaded the whitelist!",
                ["UserAllowed"] = "> The player has been added to the whitelist!",
                ["UserDisallowed"] = "> The player has been removed from the whitelist.",
                ["ArgumentsDontMeetCondition"] = "> Invalid SteamID64.",
                ["NoPermissions"] = "> You dont have enough permissions to execute this command."
            }, this);
            lang.RegisterMessages(new Dictionary<string, string> {
                ["WhitelistRejectionMessage"] = "Nie możesz wejść teraz na serwer!",
                ["NoArgsProvided"] = "> Użyj istniejących argumentów: switch, allow <steamid64> lub reload!",
                ["TurnedOn"] = "> Pomyślnie włączono whiteliste!",
                ["TurnedOff"] = "> Pomyślnie wyłączono whiteliste!",
                ["SuccessfullyReloaded"] = "> Pomyślnie przeładowano whiteliste!",
                ["UserAllowed"] = "> Pomyślnie dodano gracza do whitelisty!",
                ["UserDisallowed"] = "> Pomyślnie usunięto gracza z whitelisty!",
                ["ArgumentsDontMeetCondition"] = "> Nieprawidłowy SteamID64.",
                ["NoPermissions"] = "> Nie masz uprawnień do wykonania tej komendy."
            }, this, "pl");
        }

        [ChatCommand("whitelist")]
        private void WhitelistCommand(PlayerSession session, string command, string[] args) {
            if (permission.UserHasPermission(session.Identity.SteamId.ToString(), "gwhitelist.admin") || session.IsAdmin) {
                if (args.Length >= 1) {
                    switch (args[0]) {
                        case "switch":
                            if ((bool)dataFile["Enabled"]) {
                                dataFile["Enabled"] = false;
                                hurt.SendChatMessage(session, null, lang.GetMessage("TurnedOff", this, session.Identity.SteamId.ToString()));
                            } else if ((bool)dataFile["Enabled"] == false) {
                                dataFile["Enabled"] = true;
                                hurt.SendChatMessage(session, null, lang.GetMessage("TurnedOn", this, session.Identity.SteamId.ToString()));
                            } else {
                                dataFile["Enabled"] = false;
                                hurt.SendChatMessage(session, null, lang.GetMessage("TurnedOff", this, session.Identity.SteamId.ToString()));
                            }
                            dataFile.Save();
                            break;
                        case "allow":
                            Regex regex = new Regex(@"^[0-9]+$");
                            if (args.Length >= 2 && regex.IsMatch(args[1])) {
                                List<string> currentWhitelist = new List<string>(dataFile["AllowedSteamIds"].ToString().Split('/'));
                                if (currentWhitelist.Contains(args[1])) {
                                    currentWhitelist.Remove(args[1]);
                                    dataFile["AllowedSteamIds"] = string.Join("/", currentWhitelist.ToArray());
                                    hurt.SendChatMessage(session, null, lang.GetMessage("UserDisallowed", this, session.Identity.SteamId.ToString()));
                                } else {
                                    currentWhitelist.Add(args[1]);
                                    dataFile["AllowedSteamIds"] = string.Join("/", currentWhitelist.ToArray());
                                    hurt.SendChatMessage(session, null, lang.GetMessage("UserAllowed", this, session.Identity.SteamId.ToString()));
                                }
                                dataFile.Save();
                            } else {
                                hurt.SendChatMessage(session, null, lang.GetMessage("ArgumentsDontMeetCondition", this, session.Identity.SteamId.ToString()));
                            }
                            break;
                        case "reload":
                            dataFile = Interface.Oxide.DataFileSystem.GetDatafile("GWhitelist");
                            hurt.SendChatMessage(session, null, lang.GetMessage("SuccessfullyReloaded", this, session.Identity.SteamId.ToString()));
                            break;
                        default:
                            hurt.SendChatMessage(session, null, lang.GetMessage("NoArgsProvided", this, session.Identity.SteamId.ToString()));
                            break;
                    }
                } else {
                    hurt.SendChatMessage(session, null, lang.GetMessage("NoArgsProvided", this, session.Identity.SteamId.ToString()));
                }
            } else {
                hurt.SendChatMessage(session, null, lang.GetMessage("NoPermissions", this, session.Identity.SteamId.ToString()));
            }
        }

        private object CanClientLogin(PlayerSession session) {
            if ((bool)dataFile["Enabled"]) {
                List<string> currentWhitelist = new List<string>(dataFile["AllowedSteamIds"].ToString().Split('/'));
                if (currentWhitelist.Contains(session.Identity.SteamId.ToString())) {
                    return true;
                }
                return lang.GetMessage("WhitelistRejectionMessage", this, session.Identity.SteamId.ToString());
            }
            return true;
        }
    }
}
